@@ -55,7 +55,10 @@ pub mod verifier;
 use crate::bio_enrollment_command::SubCommand as BioCmd;
 use crate::bio_enrollment_params::{BioSensorInfo, EnrollStatus1, EnrollStatus2, TemplateInfo};
 use crate::client_pin_command::SubCommand as PinCmd;
+use crate::get_assertion_params::Assertion;
 use crate::get_assertion_params::Extension as Gext;
+use crate::make_credential_params::Attestation;
+use crate::make_credential_params::CredentialSupportedKeyType;
 use crate::make_credential_params::Extension as Mext;
 use crate::public_key_credential_descriptor::PublicKeyCredentialDescriptor;
 use crate::public_key_credential_user_entity::PublicKeyCredentialUserEntity;
@@ -281,11 +284,22 @@ pub fn make_credential(
     rpid: &str,
     challenge: &[u8],
     pin: Option<&str>,
-) -> Result<make_credential_params::Attestation> {
+) -> Result<Attestation> {
     let device = get_device(cfg)?;
-    make_credential::make_credential(
-        &device, rpid, challenge, pin, false, None, /*None,*/ None,
-    )
+    make_credential::make_credential(&device, rpid, challenge, pin, false, None, None, None)
+}
+
+/// Registration command. Generate credentials (with PIN, non Resident Key) while also
+/// specifying the type of key you'd like to create.
+pub fn make_credential_with_key_type(
+    cfg: &LibCfg,
+    rpid: &str,
+    challenge: &[u8],
+    pin: Option<&str>,
+    key_type: Option<CredentialSupportedKeyType>,
+) -> Result<Attestation> {
+    let device = get_device(cfg)?;
+    make_credential::make_credential(&device, rpid, challenge, pin, false, None, None, key_type)
 }
 
 pub fn make_credential_with_extensions(
@@ -294,11 +308,9 @@ pub fn make_credential_with_extensions(
     challenge: &[u8],
     pin: Option<&str>,
     extensions: Option<&Vec<Mext>>,
-) -> Result<make_credential_params::Attestation> {
+) -> Result<Attestation> {
     let device = get_device(cfg)?;
-    make_credential::make_credential(
-        &device, rpid, challenge, pin, false, None, /*None,*/ extensions,
-    )
+    make_credential::make_credential(&device, rpid, challenge, pin, false, None, extensions, None)
 }
 
 /// Registration command.Generate credentials(with PIN ,Resident Key)
@@ -308,7 +320,7 @@ pub fn make_credential_rk(
     challenge: &[u8],
     pin: Option<&str>,
     rkparam: &PublicKeyCredentialUserEntity,
-) -> Result<make_credential_params::Attestation> {
+) -> Result<Attestation> {
     let device = get_device(cfg)?;
     make_credential::make_credential(
         &device,
@@ -319,6 +331,7 @@ pub fn make_credential_rk(
         Some(rkparam),
         //None,
         None,
+        None,
     )
 }
 
@@ -327,13 +340,108 @@ pub fn make_credential_without_pin(
     cfg: &LibCfg,
     rpid: &str,
     challenge: &[u8],
-) -> Result<make_credential_params::Attestation> {
+) -> Result<Attestation> {
     let device = get_device(cfg)?;
+    make_credential::make_credential(&device, rpid, challenge, None, false, None, None, None)
+}
+
+pub fn make_credential_with_args(cfg: &LibCfg, args: &MakeCredentialArgs) -> Result<Attestation> {
+    let device = get_device(cfg)?;
+
+    let extensions = if args.extensions.is_some() {
+        Some(args.extensions.as_ref().unwrap())
+    } else {
+        None
+    };
+
+    let (rk, rk_param) = if args.rkparam.is_some() {
+        (true, Some(args.rkparam.as_ref().unwrap()))
+    } else {
+        (false, None)
+    };
+
     make_credential::make_credential(
-        &device, rpid, challenge, None, false, None, /*None,*/ None,
+        &device,
+        &args.rpid,
+        &args.challenge,
+        args.pin,
+        rk,
+        rk_param,
+        extensions,
+        args.key_type,
     )
 }
 
+#[derive(Debug)]
+pub struct MakeCredentialArgs<'a> {
+    rpid: String,
+    challenge: Vec<u8>,
+    pin: Option<&'a str>,
+    key_type: Option<CredentialSupportedKeyType>,
+    extensions: Option<Vec<Mext>>,
+    rkparam: Option<PublicKeyCredentialUserEntity>,
+}
+impl<'a> MakeCredentialArgs<'a> {
+    pub fn builder() -> MakeCredentialArgsBuilder<'a> {
+        MakeCredentialArgsBuilder::default()
+    }
+}
+
+#[derive(Default)]
+pub struct MakeCredentialArgsBuilder<'a> {
+    rpid: String,
+    challenge: Vec<u8>,
+    pin: Option<&'a str>,
+    key_type: Option<CredentialSupportedKeyType>,
+    extensions: Option<Vec<Mext>>,
+    rkparam: Option<PublicKeyCredentialUserEntity>,
+}
+impl<'a> MakeCredentialArgsBuilder<'a> {
+    pub fn new(rpid: &str, challenge: &[u8]) -> MakeCredentialArgsBuilder<'a> {
+        let mut result = MakeCredentialArgsBuilder::default();
+        result.rpid = String::from(rpid);
+        result.challenge = challenge.to_vec();
+        result
+    }
+
+    pub fn pin(mut self, pin: &'a str) -> MakeCredentialArgsBuilder {
+        self.pin = Some(pin);
+        self
+    }
+
+    pub fn key_type(
+        mut self,
+        key_type: CredentialSupportedKeyType,
+    ) -> MakeCredentialArgsBuilder<'a> {
+        self.key_type = Some(key_type);
+        self
+    }
+
+    pub fn extensions(mut self, extensions: &[Mext]) -> MakeCredentialArgsBuilder<'a> {
+        self.extensions = Some(extensions.to_vec());
+        self
+    }
+
+    pub fn rkparam(
+        mut self,
+        rkparam: &PublicKeyCredentialUserEntity,
+    ) -> MakeCredentialArgsBuilder<'a> {
+        self.rkparam = Some(rkparam.clone());
+        self
+    }
+
+    pub fn build(self) -> MakeCredentialArgs<'a> {
+        MakeCredentialArgs {
+            rpid: self.rpid,
+            challenge: self.challenge,
+            pin: self.pin,
+            key_type: self.key_type,
+            extensions: self.extensions,
+            rkparam: self.rkparam,
+        }
+    }
+}
+
 /// Authentication command(with PIN , non Resident Key)
 pub fn get_assertion(
     cfg: &LibCfg,
@@ -341,19 +449,11 @@ pub fn get_assertion(
     challenge: &[u8],
     credential_id: &[u8],
     pin: Option<&str>,
-) -> Result<get_assertion_params::Assertion> {
+) -> Result<Assertion> {
     let device = get_device(cfg)?;
 
-    let asss = get_assertion::get_assertion(
-        &device,
-        rpid,
-        challenge,
-        credential_id,
-        pin,
-        true,
-        //None,
-        None,
-    )?;
+    let asss =
+        get_assertion::get_assertion(&device, rpid, challenge, credential_id, pin, true, None)?;
     Ok(asss[0].clone())
 }
 
@@ -365,7 +465,7 @@ pub fn get_assertion_with_extensios(
     credential_id: &[u8],
     pin: Option<&str>,
     extensions: Option<&Vec<Gext>>,
-) -> Result<get_assertion_params::Assertion> {
+) -> Result<Assertion> {
     let device = get_device(cfg)?;
     let asss = get_assertion::get_assertion(
         &device,
@@ -386,12 +486,94 @@ pub fn get_assertions_rk(
     rpid: &str,
     challenge: &[u8],
     pin: Option<&str>,
-) -> Result<Vec<get_assertion_params::Assertion>> {
+) -> Result<Vec<Assertion>> {
     let device = get_device(cfg)?;
     let dmy: [u8; 0] = [];
-    get_assertion::get_assertion(
-        &device, rpid, challenge, &dmy, pin, true, /*None,*/ None,
-    )
+    get_assertion::get_assertion(&device, rpid, challenge, &dmy, pin, true, None)
+}
+
+#[derive(Debug)]
+pub struct GetAssertionArgs<'a> {
+    rpid: String,
+    challenge: Vec<u8>,
+    pin: Option<&'a str>,
+    credential_id: Option<Vec<u8>>,
+    extensions: Option<Vec<Gext>>,
+}
+impl<'a> GetAssertionArgs<'a> {
+    pub fn builder() -> GetAssertionArgsBuilder<'a> {
+        GetAssertionArgsBuilder::default()
+    }
+}
+
+#[derive(Default)]
+pub struct GetAssertionArgsBuilder<'a> {
+    rpid: String,
+    challenge: Vec<u8>,
+    pin: Option<&'a str>,
+    credential_id: Option<Vec<u8>>,
+    extensions: Option<Vec<Gext>>,
+}
+impl<'a> GetAssertionArgsBuilder<'a> {
+    pub fn new(rpid: &str, challenge: &[u8]) -> GetAssertionArgsBuilder<'a> {
+        let mut result = GetAssertionArgsBuilder::default();
+        result.rpid = String::from(rpid);
+        result.challenge = challenge.to_vec();
+        result
+    }
+
+    pub fn pin(mut self, pin: &'a str) -> GetAssertionArgsBuilder {
+        self.pin = Some(pin);
+        self
+    }
+
+    pub fn extensions(mut self, extensions: &[Gext]) -> GetAssertionArgsBuilder<'a> {
+        self.extensions = Some(extensions.to_vec());
+        self
+    }
+
+    pub fn credential_id(mut self, credential_id: &[u8]) -> GetAssertionArgsBuilder<'a> {
+        self.credential_id = Some(credential_id.to_vec());
+        self
+    }
+
+    pub fn build(self) -> GetAssertionArgs<'a> {
+        GetAssertionArgs {
+            rpid: self.rpid,
+            challenge: self.challenge,
+            pin: self.pin,
+            credential_id: self.credential_id,
+            extensions: self.extensions,
+        }
+    }
+}
+pub fn get_assertion_with_args(cfg: &LibCfg, args: &GetAssertionArgs) -> Result<Vec<Assertion>> {
+    let device = get_device(cfg)?;
+
+    let credential_id = if args.credential_id.is_some() {
+        args.credential_id.as_ref().unwrap().to_vec()
+    } else {
+        let dmy: [u8; 0] = [];
+        dmy.to_vec()
+    };
+
+    let extensions = if args.extensions.is_some() {
+        Some(args.extensions.as_ref().unwrap())
+    } else {
+        None
+    };
+
+    let asss = get_assertion::get_assertion(
+        &device,
+        &args.rpid,
+        &args.challenge,
+        &credential_id,
+        args.pin,
+        true,
+        extensions,
+    )?;
+
+    Ok(asss)
 }
 
 #[derive(Debug, Clone, PartialEq)]
@@ -444,10 +626,7 @@ pub enum InfoOption {
     UserVerificationMgmtPreview,
     BioEnroll,
 }
-pub fn enable_info_option(
-    cfg: &LibCfg,
-    info_option: &InfoOption,
-) -> Result<Option<bool>> {
+pub fn enable_info_option(cfg: &LibCfg, info_option: &InfoOption) -> Result<Option<bool>> {
     let device = get_device(cfg)?;
     let info = get_info::get_info(&device)?;
     let find = match info_option {
@@ -473,9 +652,7 @@ pub fn enable_info_option(
 }
 
 /// BioEnrollment - getFingerprintSensorInfo (CTAP 2.1-PRE)
-pub fn bio_enrollment_get_fingerprint_sensor_info(
-    cfg: &LibCfg,
-) -> Result<BioSensorInfo> {
+pub fn bio_enrollment_get_fingerprint_sensor_info(cfg: &LibCfg) -> Result<BioSensorInfo> {
     let device = get_device(cfg)?;
     let init = bio_enrollment::bio_enrollment_init(&device, None).map_err(Error::msg)?;
 
@@ -595,10 +772,7 @@ pub fn bio_enrollment_cancel(cfg: &LibCfg, enroll_status: &EnrollStatus1) -> Res
 
 /// BioEnrollment - enumerateEnrollments (CTAP 2.1-PRE)
 /// 6.7.6. Enumerate enrollments
-pub fn bio_enrollment_enumerate_enrollments(
-    cfg: &LibCfg,
-    pin: &str,
-) -> Result<Vec<TemplateInfo>> {
+pub fn bio_enrollment_enumerate_enrollments(cfg: &LibCfg, pin: &str) -> Result<Vec<TemplateInfo>> {
     let device = get_device(cfg)?;
     let init = bio_enrollment::bio_enrollment_init(&device, Some(pin))?;
     let pin_token = init.1.unwrap();
@@ -793,12 +967,8 @@ pub fn credential_management_update_user_information(
 pub fn selection(cfg: &LibCfg) -> Result<String> {
     let device = get_device(cfg)?;
     let cid = ctaphid::ctaphid_init(&device).map_err(Error::msg)?;
-
     let send_payload = selection_command::create_payload();
-    println!("{}", util::to_hex_str(&send_payload));
-
     let _response_cbor = ctaphid::ctaphid_cbor(&device, &cid, &send_payload).map_err(Error::msg)?;
-
     Ok("".to_string())
 }
 
@@ -806,12 +976,8 @@ pub fn selection(cfg: &LibCfg) -> Result<String> {
 pub fn config(cfg: &LibCfg) -> Result<String> {
     let device = get_device(cfg)?;
     let cid = ctaphid::ctaphid_init(&device).map_err(Error::msg)?;
-
     let send_payload = config_command::create_payload_enable_enterprise_attestation();
-    println!("{}", util::to_hex_str(&send_payload));
-
     let _response_cbor = ctaphid::ctaphid_cbor(&device, &cid, &send_payload).map_err(Error::msg)?;
-
     Ok("".to_string())
 }
 
